@@ -1,5 +1,5 @@
 {-# LANGUAGE ExistentialQuantification, FlexibleInstances, GeneralizedNewtypeDeriving,
-             MultiParamTypeClasses, TypeSynonymInstances, CPP, DeriveDataTypeable #-}
+             MultiParamTypeClasses, TypeSynonymInstances, DeriveDataTypeable #-}
 
 -----------------------------------------------------------------------------
 -- |
@@ -36,9 +36,11 @@ import XMonad.StackSet hiding (modify)
 import Prelude
 import Control.Exception.Extensible (fromException, try, bracket, throw, finally, SomeException(..))
 import qualified Control.Exception.Extensible as E
-import Control.Applicative
+import Control.Applicative(Applicative, pure, (<$>), (<*>))
+import Control.Monad.Fail
 import Control.Monad.State
 import Control.Monad.Reader
+import Data.Semigroup
 import Data.Default
 import System.FilePath
 import System.IO
@@ -56,7 +58,8 @@ import Graphics.X11.Xlib.Extras (getWindowAttributes, WindowAttributes, Event)
 import Data.Typeable
 import Data.List ((\\))
 import Data.Maybe (isJust,fromMaybe)
-import Data.Monoid
+import Data.Monoid hiding ((<>))
+import System.Environment (lookupEnv)
 
 import qualified Data.Map as M
 import qualified Data.Set as S
@@ -71,7 +74,7 @@ data XState = XState
     , extensibleState  :: !(M.Map String (Either String StateExtension))
     -- ^ stores custom state information.
     --
-    -- The module "XMonad.Utils.ExtensibleState" in xmonad-contrib
+    -- The module "XMonad.Util.ExtensibleState" in xmonad-contrib
     -- provides additional information and a simple interface for using this.
     }
 
@@ -145,12 +148,15 @@ data ScreenDetail   = SD { screenRect :: !Rectangle } deriving (Eq,Show, Read)
 -- instantiated on 'XConf' and 'XState' automatically.
 --
 newtype X a = X (ReaderT XConf (StateT XState IO) a)
-    deriving (Functor, Monad, MonadIO, MonadState XState, MonadReader XConf, Typeable)
+    deriving (Functor, Monad, MonadFail, MonadIO, MonadState XState, MonadReader XConf, Typeable)
 
 instance Applicative X where
   pure = return
   (<*>) = ap
 
+instance Semigroup a => Semigroup (X a) where
+    (<>) = liftM2 (<>)
+
 instance (Monoid a) => Monoid (X a) where
     mempty  = return mempty
     mappend = liftM2 mappend
@@ -165,6 +171,9 @@ newtype Query a = Query (ReaderT Window X a)
 runQuery :: Query a -> Window -> X a
 runQuery (Query m) w = runReaderT m w
 
+instance Semigroup a => Semigroup (Query a) where
+    (<>) = liftM2 (<>)
+
 instance Monoid a => Monoid (Query a) where
     mempty  = return mempty
     mappend = liftM2 mappend
@@ -460,7 +469,7 @@ getXMonadDir :: MonadIO m => m String
 getXMonadDir =
     findFirstDirWithEnv "XMONAD_CONFIG_DIR"
       [ getAppUserDataDirectory "xmonad"
-      , getXdgDirectory XdgConfig "xmonad"
+      , getXDGDirectory XDGConfig "xmonad"
       ]
 
 -- | Return the path to the xmonad cache directory.  This directory is
@@ -480,7 +489,7 @@ getXMonadCacheDir :: MonadIO m => m String
 getXMonadCacheDir =
     findFirstDirWithEnv "XMONAD_CACHE_DIR"
       [ getAppUserDataDirectory "xmonad"
-      , getXdgDirectory XdgCache "xmonad"
+      , getXDGDirectory XDGCache "xmonad"
       ]
 
 -- | Return the path to the xmonad data directory.  This directory is
@@ -500,7 +509,7 @@ getXMonadDataDir :: MonadIO m => m String
 getXMonadDataDir =
     findFirstDirWithEnv "XMONAD_DATA_DIR"
       [ getAppUserDataDirectory "xmonad"
-      , getXdgDirectory XdgData "xmonad"
+      , getXDGDirectory XDGData "xmonad"
       ]
 
 -- | Helper function that will find the first existing directory and
@@ -536,6 +545,27 @@ findFirstDirWithEnv envName paths = do
       Nothing      -> findFirstDirOf paths
       Just envPath -> findFirstDirOf (return envPath:paths)
 
+-- | Helper function to retrieve the various XDG directories.
+-- This has been based on the implementation shipped with GHC version 8.0.1 or
+-- higher. Put here to preserve compatibility with older GHC versions.
+getXDGDirectory :: XDGDirectory -> FilePath -> IO FilePath
+getXDGDirectory xdgDir suffix =
+  normalise . (</> suffix) <$>
+  case xdgDir of
+    XDGData   -> get "XDG_DATA_HOME"   ".local/share"
+    XDGConfig -> get "XDG_CONFIG_HOME" ".config"
+    XDGCache  -> get "XDG_CACHE_HOME"  ".cache"
+  where
+    get name fallback = do
+      env <- lookupEnv name
+      case env of
+        Nothing -> fallback'
+        Just path
+          | isRelative path -> fallback'
+          | otherwise -> return path
+      where
+        fallback' = (</> fallback) <$> getHomeDirectory
+data XDGDirectory = XDGData | XDGConfig | XDGCache
 
 -- | Get the name of the file used to store the xmonad window state.
 stateFileName :: (Functor m, MonadIO m) => m FilePath
@@ -577,9 +607,36 @@ recompile force = io $ do
 
     useBuildscript <- do
       exists <- doesFileExist buildscript
-      if exists then isExecutable buildscript else return False
+      if exists
+        then do
+          isExe <- isExecutable buildscript
+          if isExe
+            then do
+              trace $ "XMonad will use build script at " ++ show buildscript ++ " to recompile."
+              return True
+            else do
+              trace $ unlines
+                [ "XMonad will not use build script, because " ++ show buildscript ++ " is not executable."
+                , "Suggested resolution to use it: chmod u+x " ++ show buildscript
+                ]
+              return False
+        else do
+          trace $
+            "XMonad will use ghc to recompile, because " ++ show buildscript ++ " does not exist."
+          return False
 
-    if force || useBuildscript || any (binT <) (srcT : libTs)
+    shouldRecompile <-
+      if useBuildscript || force
+        then return True
+        else if any (binT <) (srcT : libTs)
+          then do
+            trace "XMonad doing recompile because some files have changed."
+            return True
+          else do
+            trace "XMonad skipping recompile because it is not forced (e.g. via --recompile), and neither xmonad.hs nor any *.hs / *.lhs / *.hsc files in lib/ have been changed."
+            return False
+
+    if shouldRecompile
       then do
         -- temporarily disable SIGCHLD ignoring:
         uninstallSignalHandlers
@@ -592,17 +649,19 @@ recompile force = io $ do
         installSignalHandlers
 
         -- now, if it fails, run xmessage to let the user know:
-        when (status /= ExitSuccess) $ do
-            ghcErr <- readFile err
-            let msg = unlines $
-                    ["Error detected while loading xmonad configuration file: " ++ src]
-                    ++ lines (if null ghcErr then show status else ghcErr)
-                    ++ ["","Please check the file for errors."]
-            -- nb, the ordering of printing, then forking, is crucial due to
-            -- lazy evaluation
-            hPutStrLn stderr msg
-            forkProcess $ executeFile "xmessage" True ["-default", "okay", replaceUnicode msg] Nothing
-            return ()
+        if status == ExitSuccess
+            then trace "XMonad recompilation process exited with success!"
+            else do
+                ghcErr <- readFile err
+                let msg = unlines $
+                        ["Error detected while loading xmonad configuration file: " ++ src]
+                        ++ lines (if null ghcErr then show status else ghcErr)
+                        ++ ["","Please check the file for errors."]
+                -- nb, the ordering of printing, then forking, is crucial due to
+                -- lazy evaluation
+                hPutStrLn stderr msg
+                forkProcess $ executeFile "xmessage" True ["-default", "okay", replaceUnicode msg] Nothing
+                return ()
         return (status == ExitSuccess)
       else return True
  where getModTime f = E.catch (Just <$> getModificationTime f) (\(SomeException _) -> return Nothing)
